from PySide6.QtWidgets import (QMainWindow, QTextEdit, QPushButton, QVBoxLayout, QWidget,
    QLineEdit, QLabel, QComboBox, QHBoxLayout, QMessageBox)
from PySide6.QtCore import Signal, Slot
from config.settings import Settings
from responder.alarm import AlarmResponder
from responder.auto_reply import AutoReplyResponder
from responder.device_control import DeviceControlResponder

class MainWindow(QMainWindow):
    log_signal = Signal(str)

    def __init__(self, mqtt_client, parser_factory, responder):
        super().__init__()
        self.setWindowTitle("MQTT在线测试平台")
        self.resize(700, 500)
        self.mqtt_client = mqtt_client
        self.parser_factory = parser_factory
        self.responder = responder
        self.connected = False
        self.log = []

        # 配置区
        self.broker_edit = QLineEdit(Settings.MQTT_BROKER)
        self.port_edit = QLineEdit(str(Settings.MQTT_PORT))
        self.user_edit = QLineEdit(Settings.MQTT_USERNAME)
        self.pass_edit = QLineEdit(Settings.MQTT_PASSWORD)
        self.topic_edit = QLineEdit(",".join(Settings.MQTT_TOPICS))
        self.protocol_combo = QComboBox()
        self.protocol_combo.addItems(["json", "xml", "custom"])
        self.protocol_combo.setCurrentText(Settings.PROTOCOL_TYPE)
        self.responder_combo = QComboBox()
        self.responder_combo.addItems(["alarm", "auto_reply", "device_control"])
        self.responder_combo.setCurrentText(Settings.RESPONDER_TYPE)

        # 状态与日志
        self.status_label = QLabel("未连接")
        self.text_edit = QTextEdit()
        self.text_edit.setReadOnly(True)

        # 按钮
        self.btn_connect = QPushButton("连接MQTT")
        self.btn_disconnect = QPushButton("断开MQTT")
        self.btn_save = QPushButton("保存配置")
        self.btn_connect.clicked.connect(self.connect_mqtt)
        self.btn_disconnect.clicked.connect(self.disconnect_mqtt)
        self.btn_save.clicked.connect(self.save_config)

        # 布局
        form_layout = QHBoxLayout()
        form_layout.addWidget(QLabel("Broker:"))
        form_layout.addWidget(self.broker_edit)
        form_layout.addWidget(QLabel("端口:"))
        form_layout.addWidget(self.port_edit)
        form_layout.addWidget(QLabel("用户名:"))
        form_layout.addWidget(self.user_edit)
        form_layout.addWidget(QLabel("密码:"))
        form_layout.addWidget(self.pass_edit)
        form_layout.addWidget(QLabel("主题(逗号分隔):"))
        form_layout.addWidget(self.topic_edit)
        form_layout.addWidget(QLabel("协议类型:"))
        form_layout.addWidget(self.protocol_combo)
        form_layout.addWidget(QLabel("响应机制:"))
        form_layout.addWidget(self.responder_combo)
        form_layout.addWidget(self.btn_save)

        btn_layout = QHBoxLayout()
        btn_layout.addWidget(self.btn_connect)
        btn_layout.addWidget(self.btn_disconnect)
        btn_layout.addWidget(self.status_label)

        layout = QVBoxLayout()
        layout.addLayout(form_layout)
        layout.addLayout(btn_layout)
        layout.addWidget(self.text_edit)

        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)

        # 信号连接
        self.log_signal.connect(self.append_log)

    def connect_mqtt(self):
        self.apply_config()
        self.mqtt_client.set_on_message(self.on_message)
        self.mqtt_client.connect()
        self.connected = True
        self.status_label.setText("已连接")
        self.append_log("已连接MQTT")

    def disconnect_mqtt(self):
        self.mqtt_client.disconnect()
        self.connected = False
        self.status_label.setText("未连接")
        self.append_log("已断开MQTT")

    def on_message(self, client, userdata, msg):
        # 通过信号安全地传递到主线程
        try:
            raw = msg.payload.decode(errors='replace')
        except Exception:
            raw = str(msg.payload)
        self.log_signal.emit(f"收到原始消息: {raw}")
        try:
            parser = self.parser_factory.get_parser(Settings.PROTOCOL_TYPE)
            parsed = parser.parse(msg.payload)
            self.log_signal.emit(f"收到消息: {parsed}")
            responder = self.get_responder()
            responder.respond(parsed)
        except Exception as e:
            self.log_signal.emit(f"解析错误: {e}")

    @Slot(str)
    def append_log(self, text):
        self.log.append(text)
        self.text_edit.append(text)

    def save_config(self):
        Settings.MQTT_BROKER = self.broker_edit.text()
        Settings.MQTT_PORT = int(self.port_edit.text())
        Settings.MQTT_USERNAME = self.user_edit.text()
        Settings.MQTT_PASSWORD = self.pass_edit.text()
        Settings.MQTT_TOPICS = [t.strip() for t in self.topic_edit.text().split(",") if t.strip()]
        Settings.PROTOCOL_TYPE = self.protocol_combo.currentText()
        Settings.RESPONDER_TYPE = self.responder_combo.currentText()
        Settings.save()
        QMessageBox.information(self, "提示", "配置已保存！")

    def apply_config(self):
        self.save_config()
        # 重新设置MQTT订阅主题等
        self.mqtt_client.client.unsubscribe("#")
        for topic in Settings.MQTT_TOPICS:
            self.mqtt_client.subscribe(topic)

    def get_responder(self):
        t = Settings.RESPONDER_TYPE
        if t == "alarm":
            return AlarmResponder()
        elif t == "auto_reply":
            from responder.auto_reply import AutoReplyResponder
            return AutoReplyResponder()
        elif t == "device_control":
            from responder.device_control import DeviceControlResponder
            return DeviceControlResponder()
        else:
            return AlarmResponder() 